<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="cantor_shell" version="2">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_new"/>
    <Menu name="file_new_menu"><text>&amp;New</text>
        <ActionList name="new_worksheet_with_backend_list" />
    </Menu>
    <Action name="file_open"/>
    <Action name="file_open_recent"/>
    <Action name="file_example_download"/>
    <Action name="file_example_open"/>
    <Separator/>
    <Merge/>
    <Separator/>
    <Action name="file_close"/>
    <Action name="file_quit"/>
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Merge/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Merge/>
    <Menu name="panels"><text>Panels</text>
      <ActionList name="view_show_panel_list" />
    </Menu>
  </Menu>
  <Merge/>
  <Menu  name="settings"><text>&amp;Settings</text>
    <Action name="options_show_toolbar"/>
    <Action name="options_show_statusbar"/>
    <Merge name="show_merge"/>
    <Separator/>
    <Action name="options_configure_keybindings"/>
    <Action name="options_configure_toolbars"/>
    <Action name="options_configure"/>
    <Merge name="configure_merge"/>
    <Separator/>
    <Merge/>
  </Menu>
  <Menu name="help">
    <Merge />
  </Menu>
</MenuBar>
</kpartgui>
